==> dpsearch-4.28.tar.gz: dpsearch-4.28/src/server.c <==
/* Copyright (C) 2003-2004 Datapark corp. All rights reserved.
   Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#include "dps_config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <ctype.h>

#include "dps_common.h"
#include "dps_utils.h"
#include "dps_proto.h"
#include "dps_url.h"
#include "dps_hrefs.h"
#include "dps_server.h"
#include "dps_xmalloc.h"
#include "dps_host.h"
#include "dps_vars.h"
#include "dps_wild.h"
#include "dps_match.h"
#include "dps_db.h"
#include "dps_log.h"

#define DEFAULT_PROXY_PORT	3128
#define ERRSTRSIZ 1000

/* return values: 0 on success, non-zero on error */

__C_LINK int __DPSCALL DpsServerAdd(DPS_AGENT *A, DPS_SERVER *srv){
	int		res;
	int		add = 1;
	DPS_URL		from;
	char		*urlstr;
	DPS_SERVER	*new = NULL;
	size_t		i, len;
	DPS_SERVERLIST  S;
	DPS_ENV         *Conf = A->Conf;

	/* Copy URL to temp string    */
	/* to keep srv->url unchanged */
	len = strlen(DPS_NULL2EMPTY(srv->Match.pattern)) + 4;
	if ((urlstr = (char*)DpsMalloc(len)) == NULL) {
	  DpsLog(A, DPS_LOG_ERROR, "Can't alloc %d bytes at "__FILE__":%d", len, __LINE__);
	  return DPS_ERROR;
	}
	strcpy(urlstr, DPS_NULL2EMPTY(srv->Match.pattern));
	
	from.freeme = 0;
	DpsURLInit(&from);

	if((srv->Match.match_type==DPS_MATCH_BEGIN) && (urlstr[0])){
		int follow;  
		
		/* Check whether valid URL is passed */
		if((res=DpsURLParse(&from,urlstr))){
			switch(res){
				case DPS_URL_LONG:
				  DpsLog(A, DPS_LOG_ERROR, "URL too long");
/*					sprintf(Conf->errstr, "URL too long");*/
					break;
				case DPS_URL_BAD:
				  DpsLog(A, DPS_LOG_ERROR, "Badly formed URL");
/*					sprintf(Conf->errstr, "Badly formed URL");*/
					break;
				default:
				  DpsLog(A, DPS_LOG_ERROR, "Error while parsing URL");
				  break;
			}
			DpsURLFree(&from);
			DPS_FREE(urlstr);
			return(DPS_ERROR);
		}
		if((from.hostinfo) && (from.filename == NULL)) {
			/* Add trailing slash                    */
			/* http://localhost -> http://localhost/ */
			dps_snprintf(urlstr, len, "%s://%s%s", from.schema, from.hostinfo, DPS_NULL2EMPTY(from.path));
		}
		
		switch(follow=DpsVarListFindInt(&srv->Vars,"Follow",DPS_FOLLOW_PATH)){
			char * s, * anchor;
			case DPS_FOLLOW_PATH:
				/* Cut before '?' and after last '/' */
				if((anchor=strchr(urlstr,'?')))
					*anchor='\0';
				if((s=strrchr(urlstr,'/')))
					*(s+1)='\0';
				break;

			case DPS_FOLLOW_SITE:
				if (from.hostinfo != NULL) {
					/* Cut after hostinfo */
					dps_snprintf(urlstr, len, "%s://%s/", DPS_NULL2EMPTY(from.schema), from.hostinfo);
				}else{
					/* Cut after first '/' */
					if((s=strchr(urlstr,'/')))
						*(s+1)='\0';
				}
				break;
			
			case DPS_FOLLOW_NO: 
			case DPS_FOLLOW_WORLD:
			default:
				break;
		}
		if (!strcmp(DPS_NULL2EMPTY(from.schema), "news")) {
			char *c, *cc;
			/* Cat server name to remove group names */
			/* This is because group names do not    */
			/* present in message URL                */
			c=urlstr+7;
			cc=strchr(c,'/');
			if(cc)*(cc+1)='\0';
		}
	}else
	if(srv->Match.match_type==DPS_MATCH_REGEX){
		int err;
		char regerrstr[ERRSTRSIZ]="";
		
		if(DPS_OK!=(err=DpsMatchComp(&srv->Match,regerrstr,sizeof(regerrstr)-1))){
			dps_snprintf(Conf->errstr,sizeof(Conf->errstr),"Wrong regex in config file: %s: %s", urlstr,regerrstr);
			DPS_FREE(urlstr);
			DpsURLFree(&from);
			return(DPS_ERROR);
		}
	}
	for (i = 0; i < Conf->Servers.nservers; i++) {
		if (strcmp(Conf->Servers.Server[i].Match.pattern, urlstr) == 0) {
			add = 0;
			new = &Conf->Servers.Server[i];
			break;
		}
	}

	if (add) {
		if(Conf->Servers.nservers>=Conf->Servers.mservers){
			Conf->Servers.mservers+=16;
			Conf->Servers.Server=(DPS_SERVER *)DpsXrealloc(Conf->Servers.Server,Conf->Servers.mservers*sizeof(DPS_SERVER));
			if (Conf->Servers.Server == NULL) {
			  Conf->Servers.nservers = Conf->Servers.mservers = 0;
			  DpsLog(A, DPS_LOG_ERROR, "Cant' realloc %d bytes at"__FILE__":%d", 
				 Conf->Servers.mservers * sizeof(DPS_SERVER), __LINE__);
			  return DPS_ERROR;
			}
		}
		new = &Conf->Servers.Server[Conf->Servers.nservers];
		DpsServerInit(new);
	} else {
		DPS_FREE(new->Match.pattern);
	}
	
	DpsVarListReplaceLst(&new->Vars,&srv->Vars,NULL,"*");
	
	new->Match.pattern = (char*)DpsStrdup(urlstr);
	new->Match.nomatch=srv->Match.nomatch;
	new->Match.case_sense=srv->Match.case_sense;
	new->Match.match_type=srv->Match.match_type;
	new->Match.reg = srv->Match.reg;
	new->Match.arg = srv->Match.arg;
	srv->Match.reg = NULL;
	srv->Match.arg = NULL;
	new->command = srv->command;
	new->ordre = srv->ordre;
	new->weight = srv->weight;
	new->MaxHops = srv->MaxHops;
	new->MaxDocsPerServer = srv->MaxDocsPerServer;
	
	if (add) Conf->Servers.nservers++;
	
	S.Server = new;
	res=DpsSrvAction(A, &S, DPS_SRV_ACTION_ADD);
	srv->site_id = new->site_id;
	
	DPS_FREE(urlstr);
	DpsURLFree(&from);
	return(res);
}

void DpsServerFree(DPS_SERVER *Server){
	DpsMatchFree(&Server->Match);
	DpsVarListFree(&Server->Vars);
}

void DpsServerListFree(DPS_SERVERLIST *List){
	size_t i;
	
	for(i=0;i<List->nservers;i++)
		DpsServerFree(&List->Server[i]);
	
	List->nservers=List->mservers=0;
	DPS_FREE(List->Server);
}

/* This fuction finds Server entry for given URL         */
/* and return Alias in "aliastr" if it is not NULL       */
/* "aliastr" must be big enough to store result          */
/* not more than DPS_URLSTR bytes are written to aliastr */

DPS_SERVER * DpsServerFind(DPS_AGENT *Agent, DPS_SERVERLIST *List, const char *url, int charset_id, char **aliastr) {
#define NS 10
	
	size_t		i;
	char		*robots=NULL;
	DPS_SERVER	*Res=NULL;
	char            net[32];
	
	/* If it's a robot.txt, cut to hostinfo and find result */
	if((robots=strstr(url,"/robots.txt"))){
		if(!strcmp(robots,"/robots.txt")){
			robots = (char*)DpsStrdup(url);
			robots[strlen(url)-10]='\0';
		}else{
			robots=NULL;
		}
	}
	
	net[0] = '\0';
	for(i=0;i<List->nservers;i++){
 		DPS_SERVER	*srv=&List->Server[i];
 		DPS_MATCH_PART	P[10];
 		const char	*alias=DpsVarListFindStr(&srv->Vars,"Alias",NULL);
		size_t          aliastrlen;
		int             follow = DpsVarListFindInt(&srv->Vars, "Follow", DPS_FOLLOW_PATH);

		if ((srv->Match.match_type == DPS_MATCH_SUBNET) && (*net == '\0')) {
		  DPS_CONN conn;
		  DPS_URL  *URL = DpsURLInit(NULL);

		  if (URL == NULL) continue;
			
		  if(DpsURLParse(URL, url)) {
		    DpsURLFree(URL);
		    continue;
		  }
		  conn.hostname = URL->hostname;
		  conn.port=80;
		  conn.charset_id = charset_id;
		  if (DpsHostLookup(Agent, &conn) != -1) {
			unsigned char * h;
			h=(unsigned char*)(&conn.sin.sin_addr);
			dps_snprintf(net, sizeof(net) - 1, "%d.%d.%d.%d", h[0], h[1], h[2], h[3]);
		  }
		  DpsURLFree(URL);
		}
 		
		if(follow == DPS_FOLLOW_WORLD || !DpsMatchExec(&srv->Match, url, net, 10, P) ) {
			Res=srv;
			if((aliastr != NULL) && (alias != NULL)) {
			        aliastrlen = 128 + strlen(url) + strlen(alias) + strlen(srv->Match.pattern);
				*aliastr = (char*)DpsMalloc(aliastrlen);
				if (*aliastr != NULL)
				  DpsMatchApply(*aliastr, aliastrlen, url, alias, &srv->Match, 10, P);
			}
			break;
		}
	}
	DPS_FREE(robots);
	return(Res);
}

#if 0
static int cmpserver(const void *s1,const void *s2){
	int res;
	
	if(!(res=strlen(((const DPS_SERVER*)s2)->url)-strlen(((const DPS_SERVER*)s1)->url)))
		res=(((const DPS_SERVER*)s2)->rec_id)-(((const DPS_SERVER*)s1)->rec_id);
	return(res);
}
void DpsServerListSort(DPS_SERVERLIST *List){
	/*  Long name should be found first    */
	/*  to allow different options         */
	/*  for server and it's subdirectories */
	if (List->nservers) DpsSort(List->Server, List->nservers, sizeof(DPS_SERVER), cmpserver);
}
#endif

int DpsSpiderParamInit(DPS_SPIDERPARAM *Spider){
  size_t i;
	for (i = 0; i < DPS_DEFAULT_MAX_HOPS; i++) {
	  Spider->period[i] = DPS_DEFAULT_REINDEX_TIME;
	}
	Spider->max_net_errors=DPS_MAXNETERRORS;
	Spider->read_timeout=DPS_READ_TIMEOUT;
	Spider->doc_timeout=DPS_DOC_TIMEOUT;
	Spider->maxhops=DPS_DEFAULT_MAX_HOPS;
	Spider->index=1;
	Spider->follow=DPS_FOLLOW_PATH;
	Spider->use_robots=1;
	Spider->use_clones=1;
	Spider->net_error_delay_time=DPS_DEFAULT_NET_ERROR_DELAY_TIME;
	return DPS_OK;
}
__C_LINK int __DPSCALL DpsServerInit(DPS_SERVER * srv){
	bzero((void*)srv, sizeof(*srv));
	srv->Match.match_type=DPS_MATCH_BEGIN;
	srv->weight = 1; /* default ServerWeight */
	srv->MaxHops = 255; /* default MaxHops value */
	srv->MaxDocsPerServer = (uint4)-1; /* default MaxDocsPerServer value */
	srv->ndocs = 0; /* no docs indexed */
	return(0);
}

urlid_t DpsServerGetSiteId(DPS_AGENT *Indexer, DPS_SERVER *srv, DPS_URL *url) {
  char *urlstr;
  DPS_SERVERLIST sl;
  DPS_SERVER S;
  int rc;
  int follow=DpsVarListFindInt(&srv->Vars,"Follow",DPS_FOLLOW_PATH);
  
  if ((srv->Match.match_type == DPS_MATCH_BEGIN) &&
  	(srv->Match.nomatch == 0) &&
        (follow == DPS_FOLLOW_SITE)) {
    return srv->site_id;
  }
  if((urlstr = (char*)DpsMalloc(strlen(DPS_NULL2EMPTY(url->schema)) + strlen(DPS_NULL2EMPTY(url->hostname)) + 10)) == NULL) {
    return 0;
  }
  sprintf(urlstr, "%s://%s/", DPS_NULL2EMPTY(url->schema), DPS_NULL2EMPTY(url->hostname));
  {
    register size_t i;
    for (i = 0; i < strlen(urlstr); i++) urlstr[i] = tolower(urlstr[i]);
  }
  bzero((void*)&S, sizeof(S));
  sl.Server = &S;
  S.Match.pattern     = urlstr;
  S.Match.match_type  = DPS_MATCH_BEGIN;
  S.Match.nomatch     = 0;
  S.command = 'S';
  S.ordre = srv->ordre;
  S.parent = srv->site_id;
  rc = DpsSrvAction(Indexer, &sl, DPS_SRV_ACTION_ID);
  DPS_FREE(urlstr);
  return (rc == DPS_OK) ? S.site_id : 0;
}

==> dpsearch-4.29.tar.gz: dpsearch-4.29/src/server.c <==
/* Copyright (C) 2003-2005 Datapark corp. All rights reserved.
   Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/
#include "dps_common.h"
#include "dps_utils.h"
#include "dps_proto.h"
#include "dps_url.h"
#include "dps_hrefs.h"
#include "dps_server.h"
#include "dps_xmalloc.h"
#include "dps_host.h"
#include "dps_vars.h"
#include "dps_wild.h"
#include "dps_match.h"
#include "dps_db.h"
#include "dps_log.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <ctype.h>

#define DEFAULT_PROXY_PORT	3128
#define ERRSTRSIZ 1000

/* return values: 0 on success, non-zero on error */

__C_LINK int __DPSCALL DpsServerAdd(DPS_AGENT *A, DPS_SERVER *srv){
	int		res;
	int		add = 1;
	DPS_URL		from;
	char		*urlstr;
	DPS_SERVER	*new = NULL;
	size_t		i, len;
	DPS_SERVERLIST  S;
	DPS_ENV         *Conf = A->Conf;

	/* Copy URL to temp string    */
	/* to keep srv->url unchanged */
	len = strlen(DPS_NULL2EMPTY(srv->Match.pattern)) + 4;
	if ((urlstr = (char*)DpsMalloc(len)) == NULL) {
	  DpsLog(A, DPS_LOG_ERROR, "Can't alloc %d bytes at "__FILE__":%d", len, __LINE__);
	  return DPS_ERROR;
	}
	strcpy(urlstr, DPS_NULL2EMPTY(srv->Match.pattern));
	
	from.freeme = 0;
	DpsURLInit(&from);

	if((srv->Match.match_type==DPS_MATCH_BEGIN) && (urlstr[0])){
		int follow;  
		
		/* Check whether valid URL is passed */
		if((res=DpsURLParse(&from,urlstr))){
			switch(res){
				case DPS_URL_LONG:
				  DpsLog(A, DPS_LOG_ERROR, "URL too long");
/*					sprintf(Conf->errstr, "URL too long");*/
					break;
				case DPS_URL_BAD:
				  DpsLog(A, DPS_LOG_ERROR, "Badly formed URL");
/*					sprintf(Conf->errstr, "Badly formed URL");*/
					break;
				default:
				  DpsLog(A, DPS_LOG_ERROR, "Error while parsing URL");
				  break;
			}
			DpsURLFree(&from);
			DPS_FREE(urlstr);
			return(DPS_ERROR);
		}
		if((from.hostinfo) && (from.filename == NULL)) {
			/* Add trailing slash                    */
			/* http://localhost -> http://localhost/ */
			dps_snprintf(urlstr, len, "%s://%s%s", from.schema, from.hostinfo, DPS_NULL2EMPTY(from.path));
		}
		
		switch(follow=DpsVarListFindInt(&srv->Vars,"Follow",DPS_FOLLOW_PATH)){
			char * s, * anchor;
			case DPS_FOLLOW_PATH:
				/* Cut before '?' and after last '/' */
				if((anchor=strchr(urlstr,'?')))
					*anchor='\0';
				if((s=strrchr(urlstr,'/')))
					*(s+1)='\0';
				break;

			case DPS_FOLLOW_SITE:
				if (from.hostinfo != NULL) {
					/* Cut after hostinfo */
					dps_snprintf(urlstr, len, "%s://%s/", DPS_NULL2EMPTY(from.schema), from.hostinfo);
				}else{
					/* Cut after first '/' */
					if((s=strchr(urlstr,'/')))
						*(s+1)='\0';
				}
				break;
			
			case DPS_FOLLOW_NO: 
			case DPS_FOLLOW_WORLD:
			default:
				break;
		}
		if (!strcmp(DPS_NULL2EMPTY(from.schema), "news")) {
			char *c, *cc;
			/* Cat server name to remove group names */
			/* This is because group names do not    */
			/* present in message URL                */
			c=urlstr+7;
			cc=strchr(c,'/');
			if(cc)*(cc+1)='\0';
		}
	}else
	if(srv->Match.match_type==DPS_MATCH_REGEX){
		int err;
		char regerrstr[ERRSTRSIZ]="";
		
		if(DPS_OK!=(err=DpsMatchComp(&srv->Match,regerrstr,sizeof(regerrstr)-1))){
			dps_snprintf(Conf->errstr,sizeof(Conf->errstr),"Wrong regex in config file: %s: %s", urlstr,regerrstr);
			DPS_FREE(urlstr);
			DpsURLFree(&from);
			return(DPS_ERROR);
		}
	}
	for (i = 0; i < Conf->Servers.nservers; i++) {
		if (strcmp(Conf->Servers.Server[i].Match.pattern, urlstr) == 0) {
			add = 0;
			new = &Conf->Servers.Server[i];
			break;
		}
	}

	if (add) {
		if(Conf->Servers.nservers>=Conf->Servers.mservers){
			Conf->Servers.mservers+=16;
			Conf->Servers.Server=(DPS_SERVER *)DpsXrealloc(Conf->Servers.Server,Conf->Servers.mservers*sizeof(DPS_SERVER));
			if (Conf->Servers.Server == NULL) {
			  Conf->Servers.nservers = Conf->Servers.mservers = 0;
			  DpsLog(A, DPS_LOG_ERROR, "Cant' realloc %d bytes at"__FILE__":%d", 
				 Conf->Servers.mservers * sizeof(DPS_SERVER), __LINE__);
			  return DPS_ERROR;
			}
		}
		new = &Conf->Servers.Server[Conf->Servers.nservers];
		DpsServerInit(new);
	} else {
		DPS_FREE(new->Match.pattern);
	}
	
	DpsVarListReplaceLst(&new->Vars,&srv->Vars,NULL,"*");
	
	new->Match.pattern = (char*)DpsStrdup(urlstr);
	new->Match.nomatch=srv->Match.nomatch;
	new->Match.case_sense=srv->Match.case_sense;
	new->Match.match_type=srv->Match.match_type;
	new->Match.reg = srv->Match.reg;
	new->Match.arg = srv->Match.arg;
	srv->Match.reg = NULL;
	srv->Match.arg = NULL;
	new->command = srv->command;
	new->ordre = srv->ordre;
	new->weight = srv->weight;
	new->MaxHops = srv->MaxHops;
	new->MaxDocsPerServer = srv->MaxDocsPerServer;
	
	if (add) Conf->Servers.nservers++;
	
	S.Server = new;
	res=DpsSrvAction(A, &S, DPS_SRV_ACTION_ADD);
	srv->site_id = new->site_id;
	
	DPS_FREE(urlstr);
	DpsURLFree(&from);
	return(res);
}

void DpsServerFree(DPS_SERVER *Server){
	DpsMatchFree(&Server->Match);
	DpsVarListFree(&Server->Vars);
}

void DpsServerListFree(DPS_SERVERLIST *List){
	size_t i;
	
	for(i=0;i<List->nservers;i++)
		DpsServerFree(&List->Server[i]);
	
	List->nservers=List->mservers=0;
	DPS_FREE(List->Server);
}

/* This fuction finds Server entry for given URL         */
/* and return Alias in "aliastr" if it is not NULL       */
/* "aliastr" must be big enough to store result          */
/* not more than DPS_URLSTR bytes are written to aliastr */

DPS_SERVER * DpsServerFind(DPS_AGENT *Agent, DPS_SERVERLIST *List, const char *url, int charset_id, char **aliastr) {
#define NS 10
	
	size_t		i;
	char		*robots=NULL;
	DPS_SERVER	*Res=NULL;
	char            net[32];
	
	/* If it's a robot.txt, cut to hostinfo and find result */
	if((robots=strstr(url,"/robots.txt"))){
		if(!strcmp(robots,"/robots.txt")){
			robots = (char*)DpsStrdup(url);
			robots[strlen(url)-10]='\0';
		}else{
			robots=NULL;
		}
	}
	
	net[0] = '\0';
	for(i=0;i<List->nservers;i++){
 		DPS_SERVER	*srv=&List->Server[i];
 		DPS_MATCH_PART	P[10];
 		const char	*alias=DpsVarListFindStr(&srv->Vars,"Alias",NULL);
		size_t          aliastrlen;
		int             follow = DpsVarListFindInt(&srv->Vars, "Follow", DPS_FOLLOW_PATH);

		if ((srv->Match.match_type == DPS_MATCH_SUBNET) && (*net == '\0')) {
		  DPS_CONN conn;
		  DPS_URL  *URL = DpsURLInit(NULL);

		  if (URL == NULL) continue;
			
		  if(DpsURLParse(URL, url)) {
		    DpsURLFree(URL);
		    continue;
		  }
		  bzero(&conn, sizeof(conn));
		  conn.hostname = URL->hostname;
		  conn.port=80;
		  conn.charset_id = charset_id;
		  if (DpsHostLookup(Agent, &conn) != -1) {
			unsigned char * h;
			h=(unsigned char*)(&conn.sin.sin_addr);
			dps_snprintf(net, sizeof(net) - 1, "%d.%d.%d.%d", h[0], h[1], h[2], h[3]);
		  }
		  DpsURLFree(URL);
		}
 		
		if(follow == DPS_FOLLOW_WORLD || !DpsMatchExec(&srv->Match, url, net, 10, P) ) {
			Res=srv;
			if((aliastr != NULL) && (alias != NULL)) {
			        aliastrlen = 128 + strlen(url) + strlen(alias) + strlen(srv->Match.pattern);
				*aliastr = (char*)DpsMalloc(aliastrlen);
				if (*aliastr != NULL)
				  DpsMatchApply(*aliastr, aliastrlen, url, alias, &srv->Match, 10, P);
			}
			break;
		}
	}
	DPS_FREE(robots);
	return(Res);
}

#if 0
static int cmpserver(const void *s1,const void *s2){
	int res;
	
	if(!(res=strlen(((const DPS_SERVER*)s2)->url)-strlen(((const DPS_SERVER*)s1)->url)))
		res=(((const DPS_SERVER*)s2)->rec_id)-(((const DPS_SERVER*)s1)->rec_id);
	return(res);
}
void DpsServerListSort(DPS_SERVERLIST *List){
	/*  Long name should be found first    */
	/*  to allow different options         */
	/*  for server and it's subdirectories */
	if (List->nservers) DpsSort(List->Server, List->nservers, sizeof(DPS_SERVER), cmpserver);
}
#endif

int DpsSpiderParamInit(DPS_SPIDERPARAM *Spider){
  size_t i;
	for (i = 0; i < DPS_DEFAULT_MAX_HOPS; i++) {
	  Spider->period[i] = DPS_DEFAULT_REINDEX_TIME;
	}
	Spider->max_net_errors=DPS_MAXNETERRORS;
	Spider->read_timeout=DPS_READ_TIMEOUT;
	Spider->doc_timeout=DPS_DOC_TIMEOUT;
	Spider->maxhops=DPS_DEFAULT_MAX_HOPS;
	Spider->index=1;
	Spider->follow=DPS_FOLLOW_PATH;
	Spider->use_robots=1;
	Spider->use_clones=1;
	Spider->net_error_delay_time=DPS_DEFAULT_NET_ERROR_DELAY_TIME;
	return DPS_OK;
}
__C_LINK int __DPSCALL DpsServerInit(DPS_SERVER * srv){
	bzero((void*)srv, sizeof(*srv));
	srv->Match.match_type=DPS_MATCH_BEGIN;
	srv->weight = 1; /* default ServerWeight */
	srv->MaxHops = 255; /* default MaxHops value */
	srv->MaxDocsPerServer = (dps_uint4)-1; /* default MaxDocsPerServer value */
	srv->ndocs = 0; /* no docs indexed */
	return(0);
}

urlid_t DpsServerGetSiteId(DPS_AGENT *Indexer, DPS_SERVER *srv, DPS_URL *url) {
  char *urlstr;
  DPS_SERVERLIST sl;
  DPS_SERVER S;
  int rc;
  int follow=DpsVarListFindInt(&srv->Vars,"Follow",DPS_FOLLOW_PATH);
  
  if ((srv->Match.match_type == DPS_MATCH_BEGIN) &&
  	(srv->Match.nomatch == 0) &&
        (follow == DPS_FOLLOW_SITE)) {
    return srv->site_id;
  }
  if((urlstr = (char*)DpsMalloc(strlen(DPS_NULL2EMPTY(url->schema)) + strlen(DPS_NULL2EMPTY(url->hostname)) + 10)) == NULL) {
    return 0;
  }
  sprintf(urlstr, "%s://%s/", DPS_NULL2EMPTY(url->schema), DPS_NULL2EMPTY(url->hostname));
  {
    register size_t i;
    for (i = 0; i < strlen(urlstr); i++) urlstr[i] = tolower(urlstr[i]);
  }
  bzero((void*)&S, sizeof(S));
  sl.Server = &S;
  S.Match.pattern     = urlstr;
  S.Match.match_type  = DPS_MATCH_BEGIN;
  S.Match.nomatch     = 0;
  S.command = 'S';
  S.ordre = srv->ordre;
  S.parent = srv->site_id;
  rc = DpsSrvAction(Indexer, &sl, DPS_SRV_ACTION_ID);
  DPS_FREE(urlstr);
  return (rc == DPS_OK) ? S.site_id : 0;
}
